<?php
/**
 * @version    4.1.0
 * @package    Com_AllVideoShare
 * @author     Vinoth Kumar <admin@mrvinoth.com>
 * @copyright  Copyright (c) 2012 - 2022 Vinoth Kumar. All Rights Reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace MrVinoth\Component\AllVideoShare\Site\Helper;

\defined( '_JEXEC' ) or die;

use \Joomla\CMS\Factory;
use \Joomla\CMS\MVC\Model\BaseDatabaseModel;
use \Joomla\CMS\Uri\Uri;

/**
 * Class AllVideoShareQuery.
 *
 * @since  4.1.0
 */
class AllVideoShareQuery {	

	/**
	 * Get category id by alias
	 *
	 * @param   string  $alias  Item alias
	 *
	 * @return  int
	 * 
	 * @since   4.1.0
	 */
	public static function getCategoryIdByAlias( $alias = '' ) {
		if ( empty( $alias ) ) {
			return 0;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery( true );

		$query
			->select( 'id' )
			->from( $db->quoteName( '#__allvideoshare_categories' ) )
			->where( $db->quoteName( 'slug' ) . ' = ' .  $db->quote( $alias ) );

        $db->setQuery( $query );
        $result = $db->loadResult();
		
		return ! empty( $result ) ? $result : 0;		
	}

	/**
	 * Get the category item
	 *
	 * @param   int|string  $key_value  The category Id or alias
	 * @param   string      $key_field  id or alias
	 * @param   int         $state      The category published state
	 *
	 * @return  object
	 * 
	 * @since   4.1.0
	 */
	public static function getCategory( $key_value, $key_field = 'id', $state = 1 ) {
		if ( empty( $key_value ) ) {
			return null;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery( true );

		$query
			->select( '*' )
			->from( $db->quoteName( '#__allvideoshare_categories' ) );

		if ( $key_field == 'id' ) {
			$query->where( $db->quoteName( 'id' ) . ' = ' . (int) $key_value );
		} elseif ( $key_field == 'alias' ) {
			$query->where( $db->quoteName( 'slug' ) . ' = ' . $db->quote( $key_value ) );
		}		

		if ( $state ) {
			$query->where( $db->quoteName( 'state' ) . ' = 1' );
		}

        $db->setQuery( $query );
        $item = $db->loadObject();

		return $item;
	}

	/**
	 * Get category items by parent
	 *
	 * @param   int  $id     The category Id
	 * @param   int  $state  The video published state
	 *
	 * @return  mixed
	 * 
	 * @since   4.1.0
	 */
	public static function getCategoriesByParent( $id, $state = 1 ) {
		if ( empty( $id ) ) {
			return null;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery( true );

		$query
			->select( '*' )
			->from( $db->quoteName( '#__allvideoshare_categories' ) )
			->where( $db->quoteName( 'parent' ) . ' = ' .  (int) $id );

		if ( $state ) {
			$query->where( $db->quoteName( 'state' ) . ' = 1' );
		}

        $db->setQuery( $query );
        $items = $db->loadObjectList();
		
		return $items;		
	}

	/**
	 * Get video id by alias
	 *
	 * @param   string  $alias  Item alias
	 *
	 * @return  int
	 * 
	 * @since   4.1.0
	 */
	public static function getVideoIdByAlias( $alias = '' ) {
		if ( empty( $alias ) ) {
			return 0;
		}
		
		$db = Factory::getDbo();
		$query = $db->getQuery( true );

		$query
			->select( 'id' )
			->from( $db->quoteName( '#__allvideoshare_videos' ) )
			->where( $db->quoteName( 'slug' ) . ' = ' .  $db->quote( $alias ) );

        $db->setQuery( $query );
        $result = $db->loadResult();
		
		return ! empty( $result ) ? $result : 0;		
	}

	/**
	 * Get the video item
	 *
	 * @param   int|string  $key_value  The video Id or alias
	 * @param   string      $key_field  id or alias
	 * @param   int         $state      The video published state
	 *
	 * @return  mixed
	 * 
	 * @since   4.1.0
	 */
	public static function getVideo( $key_value, $key_field = 'id', $state = 1 ) {
		if ( empty( $key_value ) ) {
			return null;
		}

		$db = Factory::getDbo();
		$query = $db->getQuery( true );

		$query
			->select( '*' )
			->from( $db->quoteName( '#__allvideoshare_videos' ) );

		if ( $key_field == 'id' ) {
			$query->where( $db->quoteName( 'id' ) . ' = ' . (int) $key_value );
		} elseif ( $key_field == 'alias' ) {
			$query->where( $db->quoteName( 'slug' ) . ' = ' . $db->quote( $key_value ) );
		}		

		if ( $state ) {
			$query->where( $db->quoteName( 'state' ) . ' = 1' );
		}

        $db->setQuery( $query );
        $item = $db->loadObject();

		return $item;
	}

	/**
	 * Get related videos
	 *
	 * @param   int  $videoId  The video Id
	 * @param   int  $catId    The category Id	 * 
	 * @param   int  $limit    The videos limit
	 *
	 * @return  mixed
	 * 
	 * @since   4.1.0
	 */
	public static function getRelatedVideos( $videoId, $catId, $limit ) {
		$db = Factory::getDbo();
		$query = $db->getQuery( true );

		$query
			->select( '*' )
			->from( $db->quoteName( '#__allvideoshare_videos' ) )
			->where( 'id != ' . (int) $videoId )
			->where( '( catid = ' . (int) $catId . '  OR  catids LIKE ' . $db->quote( '% ' . (int) $catId . ' %' ) . ' )' )			
			->where( $db->quoteName( 'state' ) . ' = 1' )
			->order( 'RAND()' );

        $db->setQuery( $query, 0, $limit );
        $items = $db->loadObjectList();
		
		return $items;		
	}		
	
}
